@@ -24,8 +24,8 @@ from Orange.widgets import widget, gui, settings
 from Orange.widgets.utils import itemmodels
 from Orange.widgets.widget import Input
 
-from Orange.widgets.visualize.owscatterplotgraph import LegendItem as SPGLegendItem,\
-    HelpEventDelegate
+from Orange.widgets.visualize.owscatterplotgraph import LegendItem as SPGLegendItem
+from Orange.widgets.visualize.utils.plotutils import HelpEventDelegate
 
 
 def selected_index(view):
@@ -140,6 +140,7 @@ class OWDistributions(widget.OWWidget):
     description = "Display value distributions of a data feature in a graph."
     icon = "icons/Distribution.svg"
     priority = 120
+    keywords = []
 
     class Inputs:
         data = Input("Data", Orange.data.Table, doc="Set the input dataset")
